#!/bin/csh -f

## Set Debug Parameter 
# DEBUG=1; erase all temporary files 
#       0; leave all temporary files 
set DEBUG=1 
 
## Set Pixel Scale (Not Used)
#set PIXSCALE=0.105  

## Parameters for Photometry
# Radius within which detect object to do photometry
set PHOTRADIUS=100
# Radius of annulus to measure sky
set PHOTAN=10
# Width of annulus to measure sky
set PHOTDAN=10

if ( $#argv != 3 ) then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name)"
    exit 1
endif

set INTABLE=$1
set NPOINT=$2
set HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
set OUTIM=$HEADER".fits"

set MLDATATBL=mldata.tbl
set MLCOMBTBL=mlcomb.tbl
set FTABLE=tmp.input.tbl 

set LDATATBL=ldata.tbl
set LDATATBLREJ=ldatar.tbl
set LDATATMPTBL=ldata.tmp.tbl
set MLDATATMPTBL=mod_ldata.tmp.tbl
set LDATA1TBL=ldata.1.tbl
set LDATA2TBL=ldata.2.tbl
set SKY1TBL=sky1.tbl
set SKY2TBL=sky2.tbl
set MSKY1TBL=msky1.tbl
set MSKY2TBL=msky2.tbl

set WEIGHTTBL=tmp.weight.tbl 
set WEIGHT1TBL=tmp.weight1.tbl 
set WEIGHT2TBL=tmp.weight2.tbl 

set PHOTTABLE=RESULTS/phot.$HEADER
set OFFSETTABLE=RESULTS/offset.$HEADER

set MASKTBL=tmp.mask
set MASKTABLE=RESULTS/mask.$HEADER
set TMPMASK=tmpoffmask

set TMPFILELIST=tmpfile.list
if( -f $TMPFILELIST ) then
  \rm $TMPFILELIST
endif

set CLLOG=log/cl`date +%y%m%d%H%M%S`.log
set CLCOM=cl.com

echo $CLCOM >> $TMPFILELIST

if( !(-e log) ) then
  mkdir log
endif
if( -f $CLLOG ) then
  rm $CLLOG
endif

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG"

echo "### Preparation ###"
echo "### Preparation ###" >> $CLLOG
# split filename and weight table
gawk '{print($1)}' $INTABLE > $FTABLE
gawk '{if(NF==1)print(1);else print(0)}' $INTABLE > $WEIGHTTBL
set DITH=20
@ DITH2 = $DITH * 2
set NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

echo "### Create Offset Masks ###"
echo "### Create Offset Masks ###" >> $CLLOG
gawk '{if($3<=10)print($1,$2)}' $MASKTABLE > $TMPMASK.10
gawk '{if($3==20)print($1,$2)}' $MASKTABLE > $TMPMASK.20
gawk '{if($3==30)print($1,$2)}' $MASKTABLE > $TMPMASK.30
gawk '{if($3==40)print($1,$2)}' $MASKTABLE > $TMPMASK.40
gawk '{if($3==50)print($1,$2)}' $MASKTABLE > $TMPMASK.50
gawk '{if($3>=60)print($1,$2)}' $MASKTABLE > $TMPMASK.60

cat << EOF >> $TMPFILELIST
 $TMPMASK.10 
 $TMPMASK.20 
 $TMPMASK.30 
 $TMPMASK.40 
 $TMPMASK.50 
 $TMPMASK.60 
EOF

\rm MASK/$HEADER.??.0??.tbl
./offsettbl $TMPMASK.10 $OFFSETTABLE MASK/$TMPMASK.10
./offsettbl $TMPMASK.20 $OFFSETTABLE MASK/$TMPMASK.20
./offsettbl $TMPMASK.30 $OFFSETTABLE MASK/$TMPMASK.30
./offsettbl $TMPMASK.40 $OFFSETTABLE MASK/$TMPMASK.40
./offsettbl $TMPMASK.50 $OFFSETTABLE MASK/$TMPMASK.50
./offsettbl $TMPMASK.60 $OFFSETTABLE MASK/$TMPMASK.60

cat << EOF >> $TMPFILELIST
 MASK/$TMPMASK.10.???.tbl 
 MASK/$TMPMASK.20.???.tbl 
 MASK/$TMPMASK.30.???.tbl 
 MASK/$TMPMASK.40.???.tbl 
 MASK/$TMPMASK.50.???.tbl 
 MASK/$TMPMASK.60.???.tbl 
EOF

echo "### Erase Objects ###"
echo "### Erase Objects ###" >> $CLLOG

sed 's/sky1/msky1/g' $SKY1TBL > $MSKY1TBL
if( $NFRAME > 1 ) then
  sed 's/sky2/msky2/g' $SKY2TBL > $MSKY2TBL
endif
cat << EOF > $CLCOM
 imdel @$MLDATATBL
 imdel @$MSKY1TBL
 imdel @$MSKY2TBL
 imarith @$SKY1TBL - sky1 @$SKY1TBL
 imarith @$SKY2TBL - sky2 @$SKY2TBL
 logout
EOF
cat $CLCOM | cl >> $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

set i=1
while($i <= $NPOINT)
  echo "# Erasing Objects in Position $i #"
  echo "# Erasing Objects in Position $i #" >> $CLLOG
  set filename1=`sed -n ${i}p $SKY1TBL`
  set filename2=`sed -n ${i}p $MSKY1TBL`
  set POINTNUM=`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`
  if(-f $MASKTBL) then 
    \rm $MASKTBL
  endif
  # make list of mask table files
  ls MASK/$TMPMASK.??.$POINTNUM.tbl > $MASKTBL
  echo "eraseobj $filename1 $filename2 @$MASKTBL" > $CLCOM
  if( $NFRAME > 1 ) then
    set filename1=`sed -n ${i}p $SKY2TBL`
    set filename2=`sed -n ${i}p $MSKY2TBL`
    echo "eraseobj $filename1 $filename2 @$MASKTBL" >> $CLCOM
  endif
  echo "logout" >> $CLCOM 
  cat $CLCOM | cl >>& $CLLOG
  if ( $status != 0 ) then
   echo "# Error in IRAF ... Exiting #"
   echo "# Refer to IRAF log $CLLOG. #" 
   exit 1
  endif
  @ i += 1
end
echo $LDATATMPTBL >> $TMPFILELIST
echo $MLDATATMPTBL >> $TMPFILELIST

echo "# Adding Sky Again #"
echo "# Adding Sky Again #" >> $CLLOG
echo "imdel @$SKY1TBL" > $CLCOM
echo "imdel @$SKY2TBL" >> $CLCOM
echo "imarith @$MSKY1TBL + sky1 @$SKY1TBL" >> $CLCOM
if ( $NFRAME > 1 ) then
  echo "imarith @$MSKY2TBL + sky2 @$SKY2TBL" >> $CLCOM
endif
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

echo "### Re-Create Sky Frames ###"
echo "### Re-Create Sky Frames ###" >> $CLLOG
echo "imdel noobjsky1,noobjsky2" > $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif
echo "# Combine Noobj Sky #"
echo "imcombine @$SKY1TBL noobjsky1 combine=median reject=sigclip outtype=real lsigma=2 hsigma=2" > $CLCOM
if ( $NFRAME > 1 ) then 
  echo "imcombine @$SKY2TBL noobjsky2 combine=median reject=sigclip outtype=real lsigma=2 hsigma=2" >> $CLCOM
endif
echo "logout" >> $CLCOM 
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
cat <<EOF > $CLCOM
 imdel RESULTS/$HEADER.noobjsky1.fits
 imdel RESULTS/$HEADER.noobjsky2.fits
 imcopy noobjsky1.imh RESULTS/$HEADER.noobjsky1.fits
 imcopy noobjsky2.imh RESULTS/$HEADER.noobjsky2.fits
 logout
EOF
cat $CLCOM | cl >>& $CLLOG 
if ( $status != 0 ) then 
 echo "# Error in IRAF ... Exiting #" 
 echo "# Refer to IRAF log $CLLOG. #"  
 exit 1 
endif 

echo "Sky Image is "RESULTS/$HEADER.noobjsky1.fits
echo "             "RESULTS/$HEADER.noobjsky2.fits

echo "IRAF log was written in $CLLOG"

#echo "imdel @ldata.tbl" > cl.com
#echo "imdel @mldata.tbl" >> cl.com
#echo "imdel noobjsky1.imh" >> cl.com
#echo "imdel noobjsky2.imh" >> cl.com
#echo "logout" >> cl.com
#cat cl.com | cl
